import unittest
from .author import Author
from .category import Category
from .tag import Tag
from .content_field import ContentField
from .feed import Feed
from .page import Page
from .post import Post
auth_token = 'f97d131d955f48af0769a4c827bb47728cbd5d05'


class TestAPI(unittest.TestCase):
    def is_ok_request(self, response):
        try:
            # Check for successful api state
            self.assertIn('data', response)
        except AssertionError:
            # Check for error api state
            self.assertIn('detail', response)


class TestAuthor(TestAPI):
    def test_all(self):
        author = Author(auth_token)
        response = author.all()
        self.is_ok_request(response)

    def test_all_include(self):
        author = Author(auth_token)
        response = author.all(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for post in response['data']:
            self.assertIn('recent_posts', post)

    def test_get(self):
        author = Author(auth_token)
        response = author.get('adam-yala')
        self.is_ok_request(response)

    def test_get_include(self):
        author = Author(auth_token)
        response = author.get(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for post in response['data']:
            self.assertIn('recent_posts', post)


class TestCategory(TestAPI):
    def test_all(self):
        category = Category(auth_token)
        response = category.all()
        self.is_ok_request(response)

    def test_all_include(self):
        category = Category(auth_token)
        response = category.all(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for category_post in response['data']:
            self.assertIn('recent_posts', category_post)

    def test_get(self):
        category = Category(auth_token)
        response = category.get('beer')
        self.is_ok_request(response)

    def test_get_include(self):
        category = Category(auth_token)
        response = category.get(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for post in response['data']:
            self.assertIn('recent_posts', post)


class TestTag(TestAPI):
    def test_all(self):
        tag = Tag(auth_token)
        response = tag.all()
        self.is_ok_request(response)

    def test_all_include(self):
        tag = Tag(auth_token)
        response = tag.all(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for tag_post in response['data']:
            self.assertIn('recent_posts', tag_post)

    def test_get(self):
        tag = Tag(auth_token)
        response = tag.get('beer')
        self.is_ok_request(response)

    def test_get_include(self):
        tag = Tag(auth_token)
        response = tag.get(params={'include':'recent_posts'})
        self.is_ok_request(response)
        for post in response['data']:
            self.assertIn('recent_posts', post)


class TestContentField(TestAPI):
    def test_get(self):
        content_field = ContentField(auth_token)
        response = content_field.get(['homepage_headline', 'homepage_title'])
        self.is_ok_request(response)


class TestFeed(TestAPI):
    def test_get_rss(self):
        feed = Feed(auth_token)
        response = feed.get('rss')
        self.is_ok_request(response)

    def test_get_atom(self):
        feed = Feed(auth_token)
        response = feed.get('atom')
        self.is_ok_request(response)

    def test_get_sitemap(self):
        feed = Feed(auth_token)
        response = feed.get('sitemap')
        self.is_ok_request(response)


class TestPage(TestAPI):
    def test_get(self):
        page = Page(auth_token)
        response = page.get('news', 'hello-world')
        self.is_ok_request(response)

    def test_search(self):
        page = Page(auth_token)
        response = page.search('test')
        self.is_ok_request(response)
        response = page.search('test', {'page': 1, 'page_size': 10})
        self.is_ok_request(response)


class TestPost(TestAPI):
    def test_all(self):
        post = Post(auth_token)
        response = post.all()
        self.is_ok_request(response)
        response = post.all({'page': 1, 'page_size': 10})
        self.is_ok_request(response)

    def test_search(self):
        post = Post(auth_token)
        response = post.search('test')
        self.is_ok_request(response)
        response = post.search('test', {'page': 1, 'page_size': 10})
        self.is_ok_request(response)

    def test_get(self):
        post = Post(auth_token)
        response = post.get('test-post')
        self.is_ok_request(response)


if __name__ == '__main__':
    unittest.main()



from .author import Author
from .category import Category
from .content_field import ContentField
from .feed import Feed
from .page import Page
from .post import Post
from .tag import Tag


class ButterCMS(object):
    """ButterCMS"""
    def __init__(self, auth_token):
        self.authors = Author(auth_token)
        self.categories = Category(auth_token)
        self.tags = Tag(auth_token)
        self.content_fields = ContentField(auth_token)
        self.feeds = Feed(auth_token)
        self.pages = Page(auth_token)
        self.posts = Post(auth_token)



from .client import Client


class ContentField(Client):
    """ContentField"""
    def __init__(self, auth_token):
        Client.__init__(self, auth_token)
        self.path = 'content/'

    def get(self, keys=None, params=None):
        if not params:
            params = {}
        params['keys'] = ','.join(keys)
        return self.api_get(params=params)



from .client import Client


class Post(Client):
    """Post"""
    def __init__(self, auth_token):
        Client.__init__(self, auth_token)
        self.path = 'posts/'

    def all(self, params=None):
        return self.api_get(params=params)

    def get(self, slug):
        full_slug = '{}/'.format(slug)
        return self.api_get(slug=full_slug)

    def search(self, query, params=None):
        slug = 'search/'
        if not params:
            params = {
                'query': query,
            }
        else:
            params['query'] = query
        return self.api_get(slug=slug, params=params)



import requests

from .__version__ import __version__


class Client(object):
    """Client"""
    def __init__(self, auth_token):
        self.auth_token = auth_token
        self.url = 'https://api.buttercms.com/v2/'

    def api_get(self, slug='', params=None, path_override=None):
        payload = {
            'auth_token': self.auth_token,
        }
        if params:
            payload.update(params)

        headers = {
            'X-Butter-Client': 'Python/{}'.format(__version__),
            'Accept-Encoding': 'gzip',
        }

        if path_override:
            path = path_override
        else:
            path = self.path

        response = requests.get(
            url=self.url + path + str(slug),
            params=payload,
            headers=headers,
        )
        return response.json()

    def get(self, slug='', params=None):
        return self.api_get(slug=slug, params=params)



from .client import Client


class Category(Client):
    """Category"""
    def __init__(self, auth_token):
        Client.__init__(self, auth_token)
        self.path = 'categories/'

    def all(self, params=None):
        return self.api_get(params=params)



import sys

from butter_cms.__version__ import __version__

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.md', 'r') as f:
    readme = f.read()


install_requires = []
if sys.version_info < (2, 7, 9):
    raise Exception('ButterCMS uses the requests library to securely talk to https://buttercms.com '
        'which requires Python 2.7.9 or later.\n'
        'Please take a few seconds to upgrade to Python 2.7.9 and try again.\n'
        'https://www.python.org/downloads/')
    # TODO: Add support for < Python 2.7.9
    # install_requires.append('requests[security]')
else:
    install_requires.append('requests')


setup(
    name = 'buttercms-python',
    packages = ['butter_cms'],
    version = __version__,
    description = 'API First Blogging and CMS platform built for developers',
    long_description=readme,
    long_description_content_type="text/markdown",
    author = 'Adam Yala',
    author_email = 'adam@adamyala.com',
    url = 'https://github.com/buttercms/buttercms-python',
    download_url = 'https://github.com/buttercms/buttercms-python/tarball/0.1',
    py_modules=['butter_cms'],
    install_requires=install_requires,
    keywords = ['foo', 'bar', 'baz'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
    ]
)
